# ordinal

Get ordinals from numbers

```py
from ordinal import ordinal
num = ordinal(42)
print(num) # 42nd
```

## Testing

Clone the app and run the following:

```console
python setup.py pytest
```
